<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet
	xmlns="http://www.w3.org/1999/xhtml"
	xmlns:xs="http://www.w3.org/2001/XMLSchema"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:seaf="http://seaf.sf.net/seaf"
	xmlns:per="http://seaf.sf.net/seaf-persistence"
	version="2.0">
<xsl:import href="functions/SeafPersistenceBusinessFunctions.xsl" />

<xsl:param name="application-name" />
<xsl:param name="version" />

<xsl:variable name="title">
	<xsl:text />Persistence of <xsl:value-of select="$application-name"/>
	<xsl:text /> v. <xsl:value-of select="$version"/>
</xsl:variable>

<xsl:template match="/">
	<html>
		<head>
			<title><xsl:value-of select="$title"/></title>
			<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
			<style type="text/css">
.label {font-weight:bold;}
table {border-spacing:2; border-collapse:collapse;}
table, td {border:1px solid black;}
td {padding:2 4;}
			</style>
		</head>
	<body>
		<h1><xsl:value-of select="$title"/></h1>
		<xsl:call-template name="module-list-template" />
		<h2>List of entities</h2>
		<xsl:call-template name="entity-list-template" />
		<xsl:apply-templates select="//seaf:module" />
		<xsl:apply-templates select=".//per:entity | .//per:enum-def" />
	</body>
	</html>
</xsl:template>

<xsl:template name="module-list-template">
	<h2>List of modules</h2>
	<table>
		<tr><td>Name</td></tr>
	<xsl:for-each select="//seaf:module">
		<tr><td><xsl:copy-of select="per:link('module',@name)" /></td></tr>
	</xsl:for-each>
	</table>
</xsl:template>

<xsl:template match="seaf:module">
	<xsl:text>
	</xsl:text>
	<a name="module_{@name}">
	<h2>Module: <xsl:value-of select="@name" /></h2></a>
	<span class="label">Package:</span><xsl:text> </xsl:text>
		<xsl:value-of select="@package" /><br />
	<span class="label">Module entities:</span>
	<xsl:call-template name="entity-list-template" />
</xsl:template>

<xsl:template name="entity-list-template">
	<table>
		<tr><td>Name</td><td>Type</td><td>Description</td></tr>
	<xsl:for-each select=".//per:entity | .//per:enum-def">
		<tr>
			<td><xsl:copy-of select="per:link('entity',@name)" /></td>
			<td><xsl:call-template name="entity-type-template" /></td>
			<td><xsl:value-of select="per:description[1]"/></td>
		</tr>
	</xsl:for-each>
	</table>
</xsl:template>

<xsl:template match="per:entity | per:enum-def">
	<xsl:variable name="entity" select="@name" />
	<xsl:text>
	</xsl:text>
	<a name="entity_{@name}">
	<h2>Entity: <xsl:value-of select="@name" /></h2></a>
	<span class="label">Module:</span><xsl:text> </xsl:text>
		<xsl:copy-of select="per:link('module',ancestor::seaf:module/@name)" /><br />
	<span class="label">Type:</span><xsl:text> </xsl:text>
		<xsl:call-template name="entity-type-template" /><br />
	<span class="label">Description:</span><xsl:text> </xsl:text>
		<xsl:call-template name="descriptions-template" /><br />
<xsl:if test="local-name()='entity'">
	<xsl:variable name="fields" select="per:entity-fields(.,())" />
	<span class="label">Fields:</span>
	<table>
		<tr><td>Name</td><td>Status</td><td>Type</td><td>Description</td></tr>
	<xsl:apply-templates select="$fields" />
	<xsl:for-each select="//per:relation[per:detail/@entity-name=$entity]">
		<xsl:variable name="status" select="per:master-status(per:master/@status)" />
		<tr>
			<td><xsl:value-of select="per:relation-property-name(per:detail,.)"/></td>
			<td title="{per:status-hint($status)}">
				<xsl:value-of select="$status" />
			</td>
			<td><xsl:copy-of select="per:link('entity',per:master/@entity-name)"/></td>
			<td>Relation to master entity.</td>
		</tr>
	</xsl:for-each>
	<xsl:for-each select="//per:relation[per:master/@entity-name=$entity]">
		<tr>
			<td><xsl:value-of select="per:relation-property-name(per:master,.)"/></td>
			<td>O</td>
			<td>
				<xsl:if test="@type='one-to-many'">List [</xsl:if>
				<xsl:copy-of select="per:link('entity',per:detail/@entity-name)"/>
				<xsl:if test="@type='one-to-many'">]</xsl:if>
			</td>
			<td>Relation to detail entity.</td>
		</tr>
	</xsl:for-each>
	</table>
	<xsl:if test="@code-table">
		<xsl:call-template name="references-template">
			<xsl:with-param name="entities" 
				select="//per:persistence/per:entity[per:field/per:code-table/@name=$entity]" />
		</xsl:call-template>
	</xsl:if>
</xsl:if>
<xsl:if test="local-name()='enum-def'">
	<span class="label">Values:</span>
	<table>
		<tr><td>Code</td><td>Description</td></tr>
	<xsl:for-each select=".//per:enum-value">
		<tr>
			<td><xsl:value-of select="@code"/></td>
			<td><xsl:value-of select="@description"/></td>
		</tr>
	</xsl:for-each>
	</table>
	<xsl:call-template name="references-template">
		<xsl:with-param name="entities" 
			select="//per:persistence/per:entity[per:field/per:enum/@name=$entity]" />
	</xsl:call-template>
</xsl:if>
</xsl:template>

<xsl:template match="per:field">
	<xsl:variable name="status" select="per:field-status(@status)" />
	<tr>
		<td><xsl:value-of select="@name"/></td>
		<td title="{per:status-hint($status)}">
			<xsl:value-of select="$status" />
		</td>
		<td><xsl:call-template name="field-type-template" /></td>
		<td><xsl:call-template name="descriptions-template" /></td>
	</tr>
</xsl:template>

<xsl:template name="references-template">
<xsl:param name="entities" as="node()*" />
	<span class="label">References:</span>
	<xsl:for-each select="$entities">
		<xsl:text> </xsl:text>
		<xsl:copy-of select="per:link('entity',@name)"/>
		<xsl:if test="not(position()=last())"><xsl:text>,</xsl:text></xsl:if>
	</xsl:for-each>
</xsl:template>

<xsl:template name="descriptions-template">
	<xsl:for-each select="per:description">
		<xsl:value-of select="." />
		<xsl:if test="not(position()=last())"><br /></xsl:if>
	</xsl:for-each>
</xsl:template>

<xsl:template name="entity-type-template">
	<xsl:choose>
		<xsl:when test="local-name()='enum-def'">Enumeration</xsl:when>
		<xsl:when test="@code-table='code-based'">Code-based code table</xsl:when>
		<xsl:when test="@code-table='time-bound'">Time-bound code table</xsl:when>
		<xsl:otherwise>Entity</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:function name="per:field-status" as="xs:string">
<xsl:param name="status" />
	<xsl:choose>
		<xsl:when test="$status='mandatory'">M</xsl:when>
		<xsl:when test="$status='unique'">U</xsl:when>
		<xsl:when test="$status='transient'">T</xsl:when>
		<xsl:when test="$status='criteria-only'">CO</xsl:when>
		<xsl:when test="$status='version'">V</xsl:when>
		<xsl:when test="$status='primary-key-assigned'">PKa</xsl:when>
		<xsl:when test="$status='primary-key-generated'">PKg</xsl:when>
		<xsl:otherwise>O</xsl:otherwise>
	</xsl:choose>
</xsl:function>

<xsl:function name="per:status-hint" as="xs:string">
<xsl:param name="status" />
	<xsl:choose>
		<xsl:when test="$status='M'">Mandatory</xsl:when>
		<xsl:when test="$status='U'">Unique</xsl:when>
		<xsl:when test="$status='T'">Transient</xsl:when>
		<xsl:when test="$status='CO'">Criteria-only</xsl:when>
		<xsl:when test="$status='V'">Version</xsl:when>
		<xsl:when test="$status='PKa'">Primary key assigned</xsl:when>
		<xsl:when test="$status='PKg'">Primary key generated</xsl:when>
		<xsl:when test="$status='O'">Optional</xsl:when>
		<xsl:otherwise><xsl:text></xsl:text></xsl:otherwise>
	</xsl:choose>
</xsl:function>

<xsl:function name="per:master-status" as="xs:string">
<xsl:param name="status" />
	<xsl:choose>
		<xsl:when test="not($status='optional')">M</xsl:when>
		<xsl:otherwise>O</xsl:otherwise>
	</xsl:choose>
</xsl:function>

<xsl:template name="field-type-template">
	<xsl:choose>
		<xsl:when test="per:string">
			<xsl:text>String [</xsl:text>
			<xsl:value-of select="per:string/@length" />
			<xsl:text>]</xsl:text>
		</xsl:when>
		<xsl:when test="per:int">Integer</xsl:when>
		<xsl:when test="per:long">Long</xsl:when>
		<xsl:when test="per:float">Float</xsl:when>
		<xsl:when test="per:double">Double</xsl:when>
		<xsl:when test="per:boolean">Boolean</xsl:when>
		<xsl:when test="per:date">Date</xsl:when>
		<xsl:when test="per:time">Time</xsl:when>
		<xsl:when test="per:datetime">Date time</xsl:when>
		<xsl:when test="per:timestamp">Timestamp</xsl:when>
		<xsl:when test="per:enum">
			<xsl:text>Enum [</xsl:text>
			<xsl:copy-of select="per:link('entity',per:enum/@name)" />
			<xsl:text>]</xsl:text>
		</xsl:when>
		<xsl:when test="per:code-table">
			<xsl:text>Code table [</xsl:text>
			<xsl:copy-of select="per:link('entity',per:code-table/@name)" />
			<xsl:text>]</xsl:text>
		</xsl:when>
		<xsl:when test="per:text">Text</xsl:when>
		<xsl:when test="per:binary">Binary</xsl:when>
		<xsl:otherwise>
			<xsl:message terminate="yes">
Unknown field type on <xsl:value-of select="concat(../@name,'.',@name)" />
			</xsl:message>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:function name="per:link" as="node()*">
<xsl:param name="type" as="xs:string" />
<xsl:param name="name" as="xs:string" />
	<a href="#{$type}_{$name}"><xsl:value-of select="$name" /></a>
</xsl:function>

</xsl:stylesheet>
